@@ -11,13 +11,10 @@
  * Tcl8 language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_tcl8_cxx[] = "$Id: tcl8.cxx 12830 2011-10-30 21:51:50Z wsfulton $";
-
 #include "swigmod.h"
 #include "cparse.h"
-static int treduce = SWIG_cparse_template_reduce(0);
 
-static const char *usage = (char *) "\
+static const char *usage = "\
 Tcl 8 Options (available with -tcl)\n\
      -itcl           - Enable ITcl support\n\
      -nosafe         - Leave out SafeInit module function.\n\
@@ -185,7 +182,7 @@ public:
     /* If shadow classing is enabled, we're going to change the module name to "_module" */
     if (itcl) {
       String *filen;
-      filen = NewStringf("%s%s.itcl", Swig_file_dirname(outfile), module);
+      filen = NewStringf("%s%s.itcl", SWIG_output_directory(), module);
 
       Insert(module, 0, "_");
 
@@ -248,7 +245,6 @@ public:
 
     if (itcl) {
       Printv(f_shadow, f_shadow_stubs, "\n", NIL);
-      Close(f_shadow);
       Delete(f_shadow);
     }
 
@@ -259,7 +255,6 @@ public:
     Delete(f_header);
     Delete(f_wrappers);
     Delete(f_init);
-    Close(f_begin);
     Delete(f_runtime);
     Delete(f_begin);
     return SWIG_OK;
@@ -748,6 +743,7 @@ public:
     have_constructor = 0;
     have_destructor = 0;
     destructor_action = 0;
+    constructor_name = 0;
 
     if (itcl) {
       constructor = NewString("");
@@ -944,7 +940,7 @@ public:
 	Printv(f_shadow, "  constructor { } {\n", NIL);
 	Printv(f_shadow, "    # This constructor will fail if called directly\n", NIL);
 	Printv(f_shadow, "    if { [info class] == \"::", class_name, "\" } {\n", NIL);
-	Printv(f_shadow, "      error \"No constructor for class ", class_name, (Getattr(n, "abstract") ? " - class is abstract" : ""), "\"\n", NIL);
+	Printv(f_shadow, "      error \"No constructor for class ", class_name, (Getattr(n, "abstracts") ? " - class is abstract" : ""), "\"\n", NIL);
 	Printv(f_shadow, "    }\n", NIL);
 	Printv(f_shadow, "  }\n", NIL);
       }
@@ -1206,7 +1202,8 @@ public:
       }
     }
 
-    constructor_name = NewString(Getattr(n, "sym:name"));
+    if (!have_constructor)
+      constructor_name = NewString(Getattr(n, "sym:name"));
     have_constructor = 1;
     return SWIG_OK;
   }
